import * as v from 'valibot';
import { ClaimsQuery } from '../claims-query/v-claims-query.js';
import { VpQueryUndefinedClaimSetIdError } from '../e-vp-query.js';
import { claimSetIdRegex, getIdMetadata, idRegex } from '../u-query.js';
export var CredentialQuery;
(function (CredentialQuery) {
    CredentialQuery.vMdoc = v.object({
        id: v.pipe(v.string(), v.regex(idRegex)),
        format: v.literal('mso_mdoc'),
        meta: v.optional(v.object({
            doctype_values: v.optional(v.array(v.string())),
        })),
        claims: v.optional(v.pipe(v.array(ClaimsQuery.vMdoc), v.nonEmpty())),
        claim_sets: v.optional(v.pipe(v.array(v.array(v.pipe(v.string(), v.regex(claimSetIdRegex)))), v.nonEmpty())),
    });
    const vW3cSdJwtVcBase = v.object({
        id: v.pipe(v.string(), v.regex(idRegex)),
        claims: v.optional(v.pipe(v.array(ClaimsQuery.vW3cSdJwtVc), v.nonEmpty())),
        claim_sets: v.optional(v.pipe(v.array(v.array(v.pipe(v.string(), v.regex(claimSetIdRegex)))), v.nonEmpty())),
    });
    CredentialQuery.vSdJwtVc = v.object({
        ...vW3cSdJwtVcBase.entries,
        format: v.literal('vc+sd-jwt'),
        meta: v.optional(v.object({
            vct_values: v.optional(v.array(v.string())),
        })),
    });
    CredentialQuery.vW3c = v.object({
        ...vW3cSdJwtVcBase.entries,
        format: v.picklist(['jwt_vc_json', 'jwt_vc_json-ld']),
    });
    CredentialQuery.vModel = v.variant('format', [CredentialQuery.vMdoc, CredentialQuery.vSdJwtVc, CredentialQuery.vW3c]);
    CredentialQuery.validate = (credentialQuery) => {
        claimSetIdsAreDefined(credentialQuery);
    };
})(CredentialQuery || (CredentialQuery = {}));
// --- validations --- //
const claimSetIdsAreDefined = (credentialQuery) => {
    if (!credentialQuery.claim_sets)
        return;
    const claimIds = new Set(credentialQuery.claims?.map(c => c.id));
    const undefinedClaims = [];
    for (const claim_set of credentialQuery.claim_sets) {
        for (const claim_id of claim_set) {
            const { baseId } = getIdMetadata(claim_id);
            if (!claimIds.has(baseId)) {
                undefinedClaims.push(baseId);
            }
        }
    }
    if (undefinedClaims.length > 1) {
        throw new VpQueryUndefinedClaimSetIdError({
            message: `Credential set contains undefined credential id${undefinedClaims.length === 0 ? '' : '`s'} '${undefinedClaims.join(', ')}'`,
        });
    }
};
//# sourceMappingURL=v-credential-query.js.map